#!/usr/bin/env python

import os
from dotenv import load_dotenv
from farcaster.Signer import Signer
import argparse

def main():
	load_dotenv()
	provider	= os.getenv("OP_ETH_PROVIDER") # "Your OP ETH provider endpoint"
	user_fid 	= os.getenv("USER_FID") # "User fid that will approve the signer"
	user_key 	= os.getenv("USER_PRIVATE_KEY") # "User's private key."
	app_fid 	= os.getenv("APP_FID") # "Application fid that will issue the signer."
	app_key 	= os.getenv("APP_PRIVATE_KEY") # "Application's private key"

	
	parser = argparse.ArgumentParser(prog="fario-new-signer", description="Create a new signer")
	parser.add_argument("--provider", help="OP Eth provider endpoint")
	parser.add_argument("--user_fid", help="User's fid.")
	parser.add_argument("--user_key", help="User's private key in hex.")
	parser.add_argument("--app_fid", help="Application's fid.")
	parser.add_argument("--app_key", help="Application's private key in hex.")
	args = parser.parse_args()

	provider = args.provider if args.provider else provider
	user_fid = args.user_fid if args.user_fid else user_fid
	user_key = args.user_key if args.user_key else user_key
	app_fid = args.app_fid if args.app_fid else app_fid
	app_key = args.app_key if args.app_key else app_key

	if not (provider and user_fid and user_key and app_fid and app_key):
		print('Missing parameters. Either define them in .env or as an option.')
		sys.exit(1)

	s = Signer( provider, int(user_fid), user_key, int(app_fid), app_key )

	tx_hash = s.approve_signer()
	signer_private_key = s.key
	signer_public_key = s.signer_pub()

	print(f"=== New Signer approved =====================")
	print(f"Tx: {tx_hash}")
	print(f"User Fid: {user_fid}")
	print(f"App Fid: {user_fid}")
	print(f"Signer public key: 0x{s.signer_pub().hex()}")
	print(f"Signer private key: {s.key.hex()}")
	print(f"=== MAKE SURE YOU SAVE THE PRIVATE KEY!!! ===")

if __name__ == '__main__':
	main()


#!/usr/bin/env python
import os
import sys
import base64
from blake3 import blake3
from nacl.signing import SigningKey
from farcaster.fcproto.message_pb2 import HashScheme, Message, MessageData
from protobuf_to_dict import protobuf_to_dict

import argparse

def main():
    parser = argparse.ArgumentParser(prog="fario-sign", description="Sign (or re-sign) messages uing a new signer.")
    parser.add_argument("key", type=str, help="Signer's private key")
    args = parser.parse_args()

    signer=SigningKey(bytes.fromhex(args.key[2:]))
    signer_pub_key=signer.verify_key.encode()

    for line in sys.stdin:
        m = Message.FromString(base64.b64decode(line))
        if m.hash_scheme == HashScheme.HASH_SCHEME_BLAKE3:
            data_serialized = m.data.SerializeToString()
            msg_hash = blake3(data_serialized).digest(length=20)
            msg_signature = signer.sign(msg_hash).signature
            m.signer=signer_pub_key
            m.signature=msg_signature
            m.data_bytes = data_serialized
            out = base64.b64encode(m.SerializeToString()).decode('ascii')
            print(out)
        else:
            print("HashScheme={m.hash_scheme} not supported.")
            sys.exit(1)

if __name__ == '__main__':
    main()


#!/usr/bin/env python
import os
import sys
from datetime import datetime
from dotenv import load_dotenv
from farcaster.HubService import HubService
from farcaster.fcproto.message_pb2 import MessageType, Message
import json
FARCASTER_EPOCH = 1609459200  # January 1, 2021 UTC

import base64
from fario.protobuf_to_dict import protobuf_to_dict

import argparse

def main():
	parser = argparse.ArgumentParser(prog="fario2json", description="Convert fario export to json")
	parser.add_argument("--lines", type=int, help="Only parse first <LINES>", default=0)
	args = parser.parse_args()

	print("[")
	separator = ''
	count=0
	for line in sys.stdin:
		m = Message.FromString(base64.b64decode(line))
		out = json.dumps(protobuf_to_dict(m))
		print(separator+str(out))
		if not separator:
			separator = ', '
		if args.lines and count >= args.lines:
			break
			
	print("]")

if __name__ == '__main__':
	main()


#!/usr/bin/env python
import os
import sys
from dotenv import load_dotenv
from farcaster.HubService import HubService
from farcaster.fcproto.message_pb2 import SignatureScheme, HashScheme, Embed
from farcaster import Message, FARCASTER_EPOCH
import argparse

def main():
	parser = argparse.ArgumentParser(prog="fario-cast", description="Send a cast from the command line.")
	parser.add_argument("--hub", help="Use the hub at <ADDRESS>. Ex. --hub 192.168.1.1:2283", type=str)
	parser.add_argument("--key", type=str, help="Signer's private key")
	parser.add_argument("--fid", type=int, help="User's fid")
	parser.add_argument("--timestamp", type=int, help="Cast date (unix timestamp)")
	parser.add_argument("--embed", action="append", help="Embed a URL. Can be called multiple times.")
	parser.add_argument("-v", "--verbose", help="increase output verbosity", action="store_true")
	parser.add_argument("text", type=str, help="Cast text")
	args = parser.parse_args()
	load_dotenv()
	
	# Make sure you check .env.sample to create .env
	hub_address	= args.hub if args.hub else os.getenv("FARCASTER_HUB")
	app_signer = args.key if args.key else os.getenv("APP_SIGNER_KEY")
	user_fid = args.fid if args.fid else int( os.getenv("USER_FID") )

	timestamp = args.timestamp - FARCASTER_EPOCH if args.timestamp else None
	if args.timestamp and timestamp < 0:
		print("Error: Timestamp must be after Jan 1, 2021 00:00:00 UTC.")
		sys.exit(1)

	if len(args.embed) > 2:
		print("Error: More than 2 embeds.")
		sys.exit(1)		
	hub = HubService(hub_address, use_async=False)

	message_builder = Message.MessageBuilder(
		HashScheme.HASH_SCHEME_BLAKE3, 
		SignatureScheme.SIGNATURE_SCHEME_ED25519, 
		bytes.fromhex(app_signer[2:])
	)
	
	data = message_builder.cast.add(
		fid = user_fid, 
		text = args.text[:320],
		embeds = [ Embed(url=e) for e in args.embed ],
		timestamp = timestamp
	)
	msg  = message_builder.message(data)

	ret  = hub.SubmitMessage(msg)
	if args.verbose:
		print(f"Message posted: {args.text[:320]}")

if __name__ == '__main__':
	main()


#!/usr/bin/env python

import os
import sys
import datetime
import base64
from dotenv import load_dotenv
from farcaster.Signer import Signer, removeSigner
from nacl.signing import SigningKey
from blake3 import blake3
from farcaster.fcproto.message_pb2 import HashScheme, Message, MessageData

import argparse
from . __about__ import version

def signer_add(args):
    load_dotenv()
    provider = args.provider if args.provider else os.getenv("OP_ETH_PROVIDER")
    user_fid = args.user_fid if args.user_fid else os.getenv("USER_FID")
    user_key = args.user_key if args.user_key else os.getenv("USER_PRIVATE_KEY")
    app_fid = args.app_fid if args.app_fid else os.getenv("APP_FID")
    app_key = args.app_key if args.app_key else os.getenv("APP_PRIVATE_KEY")

    if not (provider and user_fid and user_key and app_fid and app_key):
        if not args.raw:
            print('Missing parameters. Either define them in .env or as an option.')
        sys.exit(1)

    s = Signer( provider, int(user_fid), user_key, int(app_fid), app_key )

    tx_hash = s.approve_signer()
    signer_private_key = s.key
    signer_public_key = s.signer_pub()

    if not args.raw:
        print(f"=== New Signer approved =====================")
        print(f"Tx: {tx_hash}")
        print(f"User Fid: {user_fid}")
        print(f"App Fid: {app_fid}")
        print(f"Signer public key: 0x{s.signer_pub().hex()}")
        print(f"Signer private key: {s.key.hex()}")
        print(f"=== MAKE SURE YOU SAVE THE PRIVATE KEY!!! ===")
    else:
        print(f"{tx_hash}\t{user_fid}\t{app_fid}\t0x{s.signer_pub().hex()}\t{s.key.hex()}" )

def signer_revoke(args):
    load_dotenv()

    provider = args.provider if args.provider else os.getenv("OP_ETH_PROVIDER")
    user_key = args.user_key if args.user_key else os.getenv("USER_PRIVATE_KEY")

    if not (provider and user_key and args.signer):
        if not args.raw:
            print('Missing parameters. Either define them in .env or as an option.')
        sys.exit(1)
        
    tx = removeSigner(provider, user_key, args.signer)
    if not args.raw:
        print(f"Signer {args.signer} removed. Tx={tx}")
    else:
        print(tx)

def signer_list(args):
    load_dotenv()
    hub_address = args.hub if args.hub else os.getenv("FARCASTER_HUB")

    if not hub_address:
        print("No hub address. Use --hub of set FARCASTER_HUB in .env.")
        sys.exit(1)
    from farcaster.HubService import HubService
    hub = HubService(hub_address, use_async=False)
    r = hub.GetOnChainSignersByFid(args.fid)
    signers = []
    fid_to_name = {}
    for evt in r.events:
        signers.append({
            "pub_key": f"0x{evt.signer_event_body.key.hex()}",
            "timestamp": evt.block_timestamp,
            "signer_fid": int(evt.signer_event_body.metadata.hex()[64:128], 16)
            })
    for s in sorted(signers, key=lambda d: d['timestamp']):
        if args.with_fnames:
            if s['signer_fid'] not in fid_to_name:
                ret = hub.GetUserNameProofsByFid(s['signer_fid'])
                name = [ proof.name for proof in ret.proofs if proof.type==1][0].decode('ascii')
                fid_to_name[s['signer_fid']] = name
            print(f"{s['pub_key']}\t{s['timestamp']}\t{s['signer_fid']}\t@{fid_to_name[s['signer_fid']]}")
        else:
            print(f"{s['pub_key']}\t{s['timestamp']}\t{s['signer_fid']}")

def signer_sign(args):
    signer=SigningKey(bytes.fromhex(args.key[2:]))
    signer_pub_key=signer.verify_key.encode()

    for line in sys.stdin:
        m = Message.FromString(base64.b64decode(line))
        if m.hash_scheme == HashScheme.HASH_SCHEME_BLAKE3:
            data_serialized = m.data.SerializeToString()
            msg_hash = blake3(data_serialized).digest(length=20)
            msg_signature = signer.sign(msg_hash).signature
            m.signer=signer_pub_key
            m.signature=msg_signature
            m.data_bytes = data_serialized
            out = base64.b64encode(m.SerializeToString()).decode('ascii')
            print(out)
        else:
            print("HashScheme={m.hash_scheme} not supported.")
            sys.exit(1)

def main():
    parser = argparse.ArgumentParser(prog='fario-signers')
    parser.add_argument('--version', action='version', version='%(prog)s v'+version)
    parser.add_argument('--raw', action='store_true', help="Output raw values, tab separated.")
    subparser = parser.add_subparsers()

    cmd_signer_add = subparser.add_parser("add", description="""Create a new signer.
        Using --raw will output tx_hash, user_fis, app_fid, signer_bublic_key,
        signer_private_key as a tab-separated list.
        """)
    cmd_signer_add.add_argument("--provider", help="OP Eth provider endpoint")
    cmd_signer_add.add_argument("--user_fid", help="User's fid.")
    cmd_signer_add.add_argument("--user_key", help="User's private key in hex.")
    cmd_signer_add.add_argument("--app_fid", help="Application's fid.")
    cmd_signer_add.add_argument("--app_key", help="Application's private key in hex.")
    cmd_signer_add.set_defaults(func=signer_add)

    cmd_signer_remove = subparser.add_parser("remove", description="""Remove a signer.
        Using --raw will output only the tx_hash.""")
    cmd_signer_remove.add_argument("--provider", help="OP Eth provider endpoint")
    cmd_signer_remove.add_argument("--user_key", help="User's private key in hex.")
    cmd_signer_remove.add_argument("signer", help="Signer's public key in hex.")
    cmd_signer_remove.set_defaults(func=signer_revoke)

    cmd_signer_list = subparser.add_parser("list", description="list signers")
    cmd_signer_list.add_argument("--hub", help="Use the hub at <HUB>. Ex. --hub 192.168.1.1:2283", type=str)
    cmd_signer_list.add_argument('--with_fnames', action='store_true', help="Display the fname of the Signer owner.")
    cmd_signer_list.add_argument("fid", type=int, help="Signers for <FID>")
    cmd_signer_list.set_defaults(func=signer_list)

    cmd_signer_sign = subparser.add_parser("sign", description="Sign (or re-sign) messages uing a new signer.")
    cmd_signer_sign.add_argument("key", type=str, help="Signer's private key")
    cmd_signer_sign.set_defaults(func=signer_sign)    

    args = parser.parse_args()

    if 'func' in args:
        args.func(args)
    

if __name__ == '__main__':
    main()



#!/usr/bin/env python
import os
from dotenv import load_dotenv
from farcaster.HubService import HubService
from farcaster.fcproto.message_pb2 import SignatureScheme, HashScheme, Embed
from farcaster import Message

import argparse

def main():
	parser = argparse.ArgumentParser(prog="fario-fid-byname", description="Get a user's fid")
	parser.add_argument("--hub", help="Use the hub at <ADDRESS>. Ex. --hub 192.168.1.1:2283", type=str)
	parser.add_argument("-v", "--verbose", help="increase output verbosity", action="store_true")
	parser.add_argument("username", type=str, help="Cast text")
	args = parser.parse_args()

	load_dotenv()
	# Make sure you check .env.sample to create .env
	hub_address	= args.hub if args.hub else os.getenv("FARCASTER_HUB")

	hub = HubService(hub_address, use_async=False)
	ret  = hub.GetUsernameProof(args.username)

	if args.verbose:
		print(f"User {args.username} is fid {ret.fid}")
	else:
		print(ret.fid)


if __name__ == '__main__':
	main()


#!/usr/bin/env python
import os
import sys
from time import sleep
from dotenv import load_dotenv
from farcaster.HubService import HubService
from farcaster.fcproto.message_pb2 import Message

import base64
import argparse

def main():
	parser = argparse.ArgumentParser(prog="fario-in", description="Send messages to Farcaster hub.")
	parser.add_argument("--hub", help="Use the hub at <ADDRESS>. Ex. --hub 192.168.1.1:2283", type=str)
	parser.add_argument("--wait", type=int, help="Wait for <WAIT> milliseconds between message submissions.", default=100)
	args = parser.parse_args()

	load_dotenv()
	hub_address = args.hub if args.hub else os.getenv("FARCASTER_HUB")

	if not hub_address:
		print("No hub address. Use --hub of set FARCASTER_HUB in .env.")
		sys.exit(1)

	hub = HubService(hub_address, use_async=False)

	for line in sys.stdin:
		m = Message.FromString(base64.b64decode(line))
		r = hub.SubmitMessage(m)
		sleep(args.wait/1000)

if __name__ == '__main__':
	main()


#!/usr/bin/env python

import os
import json

from web3 import Web3
from eth_account import Account

from dotenv import load_dotenv
from farcaster.Signer import removeSigner
import argparse

def main():
	load_dotenv()

	parser = argparse.ArgumentParser(prog="fario-remove-signer", description="Remove signer")
	parser.add_argument("--provider", help="OP Eth provider endpoint")
	parser.add_argument("--user_key", help="User's private key in hex.")
	parser.add_argument("signer", help="Signer's public key in hex.")
	args = parser.parse_args()

	provider = args.provider if args.provider else os.getenv("OP_ETH_PROVIDER")
	user_key = args.user_key if args.user_key else os.getenv("USER_PRIVATE_KEY")

	if not (provider and user_key and args.signer):
		print('Missing parameters. Either define them in .env or as an option.')
		sys.exit(1)
		
	tx = removeSigner(provider, user_key, args.signer)
	print(f"Signer {args.signer} removed. Tx={tx}")

if __name__ == '__main__':
	main()


# flake8: noqa
from . convertor import (protobuf_to_dict, TYPE_CALLABLE_MAP, dict_to_protobuf,
                                        REVERSE_TYPE_CALLABLE_MAP, datetime_to_timestamp,
                                        timestamp_to_datetime, get_field_names_and_options, FieldsMissing,
                                        validate_dict_for_required_pb_fields)



#!/usr/bin/env python
import os
import sys
from time import sleep
from dotenv import load_dotenv
from farcaster.HubService import HubService
from farcaster.fcproto.message_pb2 import MessageType, Message

import base64
import argparse

#def Subscribe(self, event_types, from_id=None)  -> HubEvent:
